/*
This is a part of the mDesktop code.

Copyright (C) 2002 Bobby G. Vinyard

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
/*
This is a part of the LiteStep Shell Source code.

Copyright (C) 1997-2002 The LiteStep Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
/****************************************************************************
16 March 2002
  - Added DesktopBackground settings center the supplied image within the 
    desktop window, image is centered, png images supported if you are using
	a png-enabled version of litestep
	(I haven't tested this, but anyone using a multiple monitor setup with the
	 desktop stretched across all windows should be able to specificy an image
	 large enough to stretch across all monitors seamlessly... YMMV...) 
  - Added !DesktopSetBackground... changes the background to the specified
    image
  - Fixed some processing in the *Desktop line parsing
  - General Code formatting
  - Added version info resource
****************************************************************************/

#include "Desktop.h"

const char szAppName[] = "DesktopBackgroundClass"; // Our window class, etc

Desktop *desktop; // The module
int numMonitors;
bool multipleSDA;

BOOL CALLBACK MultipleDesktopAreaEnumProc(HMONITOR hMonitor, HDC gdc, LPRECT prc, LPARAM lParam);
BOOL CALLBACK DesktopAreaEnumProc(HMONITOR hMonitor, HDC gdc, LPRECT prc, LPARAM lParam);
BOOL CALLBACK DesktopAreaResetEnumProc(HMONITOR hMonitor, HDC gdc, LPRECT prc, LPARAM lParam);

#ifdef _DEBUG
char szdebugBuffer[4096];
#endif // _DEBUG

//=========================================================
// Initialization and cleanup
//=========================================================

int initModuleEx(HWND ParentWnd, HINSTANCE dllInst, LPCSTR szPath)
{
	int code;

	Window::init(dllInst);

	desktop = new Desktop(ParentWnd, code);

	return code;
}


void quitModule(HINSTANCE dllInst)
{
	delete desktop;
}


//=========================================================
// Bang commands
//=========================================================

void BangDesktopSetClick(HWND caller, LPCSTR args)
{
	desktop->bangSetClick(args);
}


void BangDesktopSetArea(HWND caller, LPCSTR args)
{
	desktop->bangSetArea(args);
}


void BangDesktopSetBackground(HWND caller, LPCSTR args)
{
	desktop->bangSetBackground(args);
}

//=========================================================
// Module code
//=========================================================

//---------------------------------------------------------
// Module constructor
//---------------------------------------------------------
Desktop::Desktop(HWND parentWnd, int& code):
		Window(szAppName)
{
	int msgs[] = {LM_GETREVID, LM_REPAINT, LM_REFRESH, 0};

	screenLeft = SCREEN_LEFT;
	screenTop = SCREEN_TOP;
	screenWidth = SCREEN_WIDTH;
	screenHeight = SCREEN_HEIGHT;

	bmpBackground = NULL;
	hdcBackground = NULL;

	if (!createWindow(WS_EX_TOOLWINDOW, szAppName, WS_POPUP,
	                  screenLeft, screenTop, screenWidth, screenHeight, parentWnd))
	{

		RESOURCE_MSGBOX(hInstance, IDS_DESKTOP2_ERROR1, "Unable to create window", szAppName)

		code = 1;
		return ;
	}

	SetWindowPos(hWnd, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE | SWP_SHOWWINDOW | SWP_NOSENDCHANGING);

	SendMessage(hParent, LM_REGISTERMESSAGE, (WPARAM)hWnd, (LPARAM)msgs);
	SetWindowLong(hWnd, GWL_USERDATA, magicDWord);

	AddBangCommand("!DesktopSetClick", BangDesktopSetClick);
	AddBangCommand("!DesktopSetArea", BangDesktopSetArea);
	AddBangCommand("!DesktopSetBackground", BangDesktopSetBackground);

	setupDesktop();

	code = 0;
}


Desktop::~Desktop()
{
	int msgs[] = {LM_GETREVID, LM_REPAINT, LM_REFRESH, 0};

	SendMessage(hParent, LM_UNREGISTERMESSAGE, (WPARAM)hWnd, (LPARAM)msgs);

	RemoveBangCommand("!DesktopSetClick");
	RemoveBangCommand("!DesktopSetArea");
	RemoveBangCommand("!DesktopSetBackground");

	resetMinMax();

	destroyWindow();

	cleanDesktop();
}


// --------------------------------------------------------
// Sets maximized windows size
// --------------------------------------------------------
void Desktop::setMinMax(void)
{
	RECT r = {screenLeft, screenTop, screenWidth, screenHeight};
	multipleSDA = false;

	EnumDisplayMonitors(NULL, NULL, MultipleDesktopAreaEnumProc, 0);
	if (multipleSDA)
	{
		if (setDesktopArea)
		{
			RECT rcMain = {0, 0, GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN)};
			if (sdaRight <= 0)
			{
				r.right = rcMain.right + sdaRight;
			}
			else
			{
				r.right = sdaRight;
			}
			if (sdaBottom <= 0)
			{
				r.bottom = rcMain.bottom + sdaBottom;
			}
			else
			{
				r.bottom = sdaBottom;
			}
			SetDesktopArea(r.left, r.top, r.right, r.bottom);
		}
		return ;
	}

	if (setDesktopArea)
	{
		r.left = sdaLeft;
		r.top = sdaTop;
		if (sdaRight <= 0)
		{
			r.right = r.right + sdaRight;
		}
		else
		{
			r.right = sdaRight;
		}
		if (sdaBottom <= 0)
		{
			r.bottom = r.bottom + sdaBottom;
		}
		else
		{
			r.bottom = sdaBottom;
		}
	}
	if (setDesktopAreaMainMonitorOnly)
	{
		RECT rcMain = {0, 0, GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN)};
		if (sdaRight <= 0)
		{
			r.right = rcMain.right + sdaRight;
		}
		else
		{
			r.right = sdaRight;
		}
		if (sdaBottom <= 0)
		{
			r.bottom = rcMain.bottom + sdaBottom;
		}
		else
		{
			r.bottom = sdaBottom;
		}
		EnumDisplayMonitors(NULL, NULL, DesktopAreaEnumProc, (LPARAM)&rcMain);
	}

#ifdef _DEBUG
	sprintf(szdebugBuffer, "left: %i top: %i right: %i bottom: %i",
	          r.left, r.top, r.right, r.bottom);
	_LSDEBUG2("New Work Area", szdebugBuffer);
#endif // _DEBUG

	SetDesktopArea(r.left, r.top, r.right, r.bottom);
}


//---------------------------------------------------------
// Reset to old maximized windows size
//---------------------------------------------------------
void Desktop::resetMinMax(void)
{
	EnumDisplayMonitors(NULL, NULL, DesktopAreaResetEnumProc, 0);
}


void Desktop::setupDesktop()
{
	FILE* f;
	char szTemp[MAX_LINE_LENGTH+1];

	numMonitors = 1;

	setDesktopArea = GetRCBool("SetDesktopArea", true) != FALSE;
	setDesktopAreaMainMonitorOnly = GetRCBool("SetDesktopAreaMainMonitorOnly", true) != FALSE;
	sdaLeft = GetRCInt("SDALeft", 0);
	sdaRight = GetRCInt("SDARight", 0);
	sdaTop = GetRCInt("SDATop", 0);
	sdaBottom = GetRCInt("SDABottom", 0);

	// Command to run on display change
	szTemp[0] = '\0';
	GetRCString("DesktopOnDisplayChange", szTemp, "", MAX_LINE_LENGTH);
	bangDisplayChange = szTemp;

	szTemp[0] = '\0';
	GetRCString("DesktopBackground", szTemp, "\0", MAX_LINE_LENGTH);
	if (szTemp[0] != '\0')
	{
		bmpBackground = LoadLSImage(szTemp, NULL);
	}	

	// Set the default right click to !popup
	bangClicks.insert(mapClicks::value_type((UINT)MOD_RBUTTON, "!popup"));

	f = LCOpen(NULL);
	if (f)
	{
		char	buffer[4096];
		char	token1[4096], token2[4096], extra_text[4096];
		char*	tokens[2];

		tokens[0] = token1;
		tokens[1] = token2;

		buffer[0] = 0;

		while (LCReadNextConfig (f, "*Desktop", buffer, sizeof (buffer)))
		{
			int count;

			token1[0] = token2[0] = extra_text[0] = '\0';

			count = LCTokenize (buffer, tokens, 2, extra_text);

			if (count == 2)
			{

				UINT nModifier = 0;
				nModifier = parseLine(token2);

				mapClicks::iterator itClicks;
				itClicks = bangClicks.find(nModifier);
				if (itClicks != bangClicks.end())
				{
					bangClicks.erase(itClicks);
				}
				bangClicks.insert(mapClicks::value_type(nModifier, extra_text));
			}
		}
	}
	LCClose(f);

	setMinMax();
}

UINT Desktop::parseLine(LPSTR line)
{
	char* tmp;
	UINT nModifier = 0;

	tmp = strtok(line, "+");

	if (!strcmpi(tmp, "lbutton"))
	{
		nModifier |= MOD_LBUTTON;
	}
	else if (!strcmpi(tmp, "mbutton"))
	{
		nModifier |= MOD_MBUTTON;
	}
	else if (!strcmpi(tmp, "rbutton"))
	{
		nModifier |= MOD_RBUTTON;
	}
	else if (!strcmpi(tmp, "x1button"))
	{
		nModifier |= MOD_XBUTTON1;
	}
	else if (!strcmpi(tmp, "x2button"))
	{
		nModifier |= MOD_XBUTTON2;
	}
	else // No modifier... exit
	{
		return nModifier;
	}

	tmp = strtok(NULL, "+");

	while (tmp)
	{
		if (!strcmpi(tmp, "lbutton"))
		{
			nModifier |= MK_LBUTTON;
		}
		else if (!strcmpi(tmp, "mbutton"))
		{
			nModifier |= MK_MBUTTON;
		}
		else if (!strcmpi(tmp, "rbutton"))
		{
			nModifier |= MK_RBUTTON;
		}
		else if (!strcmpi(tmp, "x1button"))
		{
			nModifier |= MK_XBUTTON1;
		}
		else if (!strcmpi(tmp, "x2button"))
		{
			nModifier |= MK_XBUTTON2;
		}
		else if (!strcmpi(tmp, "ctrl"))
		{
			nModifier |= MK_CONTROL;
		}
		else if (!strcmpi(tmp, "shift"))
		{
			nModifier |= MK_SHIFT;
		}
		else if (!strcmpi(tmp, "alt"))
		{
			nModifier |= MK_MENU;
		}
		else if (!strcmpi(tmp, "apps"))
		{
			nModifier |= MK_APPS;
		}
		else if (!strcmpi(tmp, "lwin"))
		{
			nModifier |= MK_LWIN;
		}
		else if (!strcmpi(tmp, "rwin"))
		{
			nModifier |= MK_RWIN;
		}

		tmp = strtok(NULL, "+");
	}

	return nModifier;
}
void Desktop::cleanDesktop()
{
	bangClicks.clear();

	if (bmpBackground)
	{
		DeleteObject(bmpBackground);
	}
	if (hdcBackground)
	{
		DeleteDC(hdcBackground);
	}
}
//=========================================================
// Registered messages
//=========================================================

void Desktop::windowProc(Message& message)
{
	BEGIN_MESSAGEPROC
	//MESSAGE(onCreate,            WM_CREATE)
	//MESSAGE(onDestroy,           WM_DESTROY)
	MESSAGE(onActivate,          WM_ACTIVATE)
	MESSAGE(onDisplayChange,     WM_DISPLAYCHANGE)
	MESSAGE(onEndSession,        WM_ENDSESSION)
	MESSAGE(onEndSession,        WM_QUERYENDSESSION)
	MESSAGE(onEraseBkgnd,        WM_ERASEBKGND)
	MESSAGE(onGetRevId,          LM_GETREVID)
	MESSAGE(onKeyMessage,        WM_KEYDOWN)
	MESSAGE(onKeyMessage,        WM_KEYUP)
	MESSAGE(onKeyMessage,        WM_HOTKEY)
	MESSAGE(onMouseActivate,     WM_MOUSEACTIVATE)
	MESSAGE(onMouseButtonDown,   WM_LBUTTONDOWN)
	MESSAGE(onMouseButtonDown,   WM_MBUTTONDOWN)
	MESSAGE(onMouseButtonDown,   WM_RBUTTONDOWN)
	MESSAGE(onMouseButtonDown,   WM_XBUTTONDOWN)
	MESSAGE(onMouseButtonUp,     WM_LBUTTONUP)
	MESSAGE(onMouseButtonUp,     WM_MBUTTONUP)
	MESSAGE(onMouseButtonUp,     WM_RBUTTONUP)
	MESSAGE(onMouseButtonUp,     WM_XBUTTONUP)
	MESSAGE(onSysCommand,        WM_SYSCOMMAND)
	MESSAGE(onWindowPosChanging, WM_WINDOWPOSCHANGING)
	MESSAGE(onRefresh,           LM_REFRESH)
	END_MESSAGEPROC
}


//=========================================================
// Message handlers
//=========================================================

void Desktop::onActivate(Message& message)
{
	// Tells litestep it has been selected
	SendMessage(hParent, LM_LSSELECT, 0, 0);
	if (message.wParamLo == WA_ACTIVE || message.wParamLo == WA_CLICKACTIVE)
	{
		SetActiveWindow(hParent);
	}
}


void Desktop::onDisplayChange(Message& message)
{
	SetWindowPos(hWnd, HWND_BOTTOM, SCREEN_LEFT, SCREEN_TOP, SCREEN_WIDTH,
	             SCREEN_HEIGHT, SWP_NOMOVE | SWP_NOACTIVATE | SWP_NOSENDCHANGING);
	setMinMax();

	if (bangDisplayChange.length() > 1)
	{
		LSExecute(hWnd, bangDisplayChange.c_str(), 0);
	}
}


void Desktop::onEndSession(Message& message)
{
	message.lResult = SendMessage(hParent, message.uMsg, message.wParam, message.lParam);
}


void Desktop::onEraseBkgnd(Message& message)
{
    HDC hDC = (HDC)message.wParam;
    HBITMAP oldImage;
    int height;
    int width;
    
    PaintDesktop(hDC);
    if (bmpBackground)
    {
        GetLSBitmapSize(bmpBackground, &width, &height);
        
        if (!hdcBackground)
        {
            hdcBackground = CreateCompatibleDC(hDC);
        }
        oldImage = HBITMAP(SelectObject(hdcBackground, bmpBackground));
        BitBlt(hDC, (screenWidth - width) / 2, (screenHeight - height) / 2, screenWidth, screenHeight, hdcBackground, 0, 0, SRCCOPY);
        SelectObject(hdcBackground, oldImage);
    }
    message.lResult = TRUE;
}


void Desktop::onGetRevId(Message& message)
{
	LPSTR buf = (LPSTR)(message.lParam);

	switch (message.wParam)
	{
		case 0:
		strcpy(buf, "mDesktop.dll: 1.001");
		//sprintf(buf, "mDesktop.dll: 1.001", &rcsRevision[11]);
		buf[strlen(buf) - 1] = '\0';
		break;
		/*case 1:
		strcpy(buf, &rcsId[1]);
		buf[strlen(buf) - 1] = '\0';
		break;*/
		default:
		strcpy(buf, "");
	}
	message.lResult = strlen(buf);
}


void Desktop::onKeyMessage(Message& message)
{
	// Forward these messages
	PostMessage(hParent, message.uMsg, message.wParam, message.lParam);
}


void Desktop::onMouseActivate(Message& message)
{
	// Tells litestep it has been selected
	PostMessage(hParent, LM_LSSELECT, 0, 0);
	// Close any popup open
	//SendMessage(hParent, LM_HIDEPOPUP, (int)(message.lParamHi), (int)(message.lParamLo));
	message.lResult = MA_NOACTIVATE;
}


void Desktop::onMouseButtonDown(Message& message)
{
	SendMessage(hParent, LM_HIDEPOPUP, (int)(message.lParamHi), (int)(message.lParamLo));
}


void Desktop::onMouseButtonUp(Message& message)
{
	UINT nModifier = 0;

	switch (message.uMsg)
	{
		case WM_LBUTTONUP:
		{
			nModifier = MOD_LBUTTON;
			break;
		}
		case WM_RBUTTONUP:
		{
			nModifier = MOD_RBUTTON;
			break;
		}
		case WM_MBUTTONUP:
		{
			nModifier = MOD_MBUTTON;
			break;
		}
		case WM_XBUTTONUP:
		{
			switch (GET_XBUTTON_WPARAM(message.wParam))
			{
				case XBUTTON1:
				{
					nModifier = MOD_XBUTTON1;
					break;
				}
				case XBUTTON2:
				{
					nModifier = MOD_XBUTTON2;
					break;
				}
			}
			message.lResult = true;
			break;
		}
		default:
		{
			return ;
		}
	}
	nModifier |= message.wParam;

	if (GetKeyState(VK_APPS) < 0)
	{
		nModifier |= MK_APPS;
	}
	if (GetKeyState(VK_MENU) < 0)
	{
		nModifier |= MK_MENU;
	}
	if (GetKeyState(VK_LWIN) < 0)
	{
		nModifier |= MK_LWIN;
	}
	if (GetKeyState(VK_RWIN) < 0)
	{
		nModifier |= MK_RWIN;
	}

	mapClicks::iterator itClicks;
	itClicks = bangClicks.find(nModifier);
	if (itClicks != bangClicks.end())
	{
		LSExecute(hWnd, (*itClicks).second.c_str(), 0);
	}
}


void Desktop::onSysCommand(Message& message)
{
	if (message.wParam == SC_CLOSE)
	{
		PostMessage(hParent, WM_KEYDOWN, LM_SHUTDOWN, 0);
	}
	else
	{
		message.lResult = DefWindowProc(hWnd, message.uMsg, message.wParam, message.lParam);
	}
}


void Desktop::onWindowPosChanging(Message& message)
{
	WINDOWPOS *c = (WINDOWPOS*)(message.lParam);

	c->hwndInsertAfter = HWND_BOTTOM;
	c->flags |= SWP_NOMOVE | SWP_NOSIZE;
}

void Desktop::onRefresh(Message& message)
{
	cleanDesktop();

	resetMinMax();

	screenLeft =   SCREEN_LEFT;
	screenTop =    SCREEN_TOP;
	screenWidth =  SCREEN_WIDTH;
	screenHeight = SCREEN_HEIGHT;

	SetWindowPos(hWnd, HWND_BOTTOM, SCREEN_LEFT, SCREEN_TOP, SCREEN_WIDTH,
	             SCREEN_HEIGHT, SWP_NOMOVE | SWP_NOACTIVATE | SWP_NOSENDCHANGING);

	setupDesktop();
}

//=========================================================
// Bang command handling
//=========================================================

void Desktop::bangSetBackground(LPCSTR args)
{
    char szFile[MAX_LINE_LENGTH+1];
    HBITMAP bmpNew;

    if (args)
    {
        if (GetToken(args, szFile, NULL, FALSE))
        {
            bmpNew = LoadLSImage(szFile, NULL);
            if (bmpNew)
            {
                bmpBackground = (HBITMAP)SelectObject(hdcBackground, bmpNew);
                if (bmpBackground)
                {
                    DeleteObject(bmpBackground);
                }
                bmpBackground = bmpNew;
            }
        }
    }
}


void Desktop::bangSetClick(LPCSTR args)
{
	if (args)
	{
		char	buffer[4096];
		char	token1[4096], extra_text[4096];
		char*	tokens[1];

		tokens[0] = token1;

		strcpy(buffer, args);

		int count;

		token1[0] = extra_text[0] = '\0';

		count = LCTokenize (buffer, tokens, 1, extra_text);

		if (count == 1)
		{

			UINT nModifier = 0;
			nModifier = parseLine(token1);

			mapClicks::iterator itClicks;
			itClicks = bangClicks.find(nModifier);
			if (itClicks != bangClicks.end())
			{
				bangClicks.erase(itClicks);
			}
			bangClicks.insert(mapClicks::value_type(nModifier, extra_text));
		}
	}
}


void Desktop::bangSetArea(LPCSTR args)
{
	char token[MAX_LINE_LENGTH];
	LPCSTR nextToken = args;

	if (GetToken(nextToken, token, &nextToken, FALSE))
	{
		sdaLeft = atoi(token);
	}
	if (GetToken(nextToken, token, &nextToken, FALSE))
	{
		sdaTop = atoi(token);
	}
	if (GetToken(nextToken, token, &nextToken, FALSE))
	{
		sdaRight = atoi(token);
	}
	if (GetToken(nextToken, token, &nextToken, FALSE))
	{
		sdaBottom = atoi(token);
	}

	setMinMax();
}


//Get and set the multiple workareas
BOOL CALLBACK MultipleDesktopAreaEnumProc(HMONITOR hMonitor, HDC gdc, LPRECT prc, LPARAM lParam)
{
	char config[32];
	bool isFoundSDA = false;
	RECT rcSDA;

	// get the exec command
	sprintf( config, "SetDesktopArea%d", numMonitors );
	isFoundSDA = GetRCBool(config, true) != FALSE;
	if (isFoundSDA)
	{
		multipleSDA = true;
		sprintf( config, "SDALeft%d", numMonitors );
		rcSDA.left = GetRCInt(config, 0);
		sprintf( config, "SDARight%d", numMonitors );
		rcSDA.right = GetRCInt(config, 0);
		sprintf( config, "SDATop%d", numMonitors );
		rcSDA.top = GetRCInt(config, 0);
		sprintf( config, "SDABottom%d", numMonitors );
		rcSDA.bottom = GetRCInt(config, 0);

		prc->left += rcSDA.left;
		prc->top += rcSDA.top;
		if (rcSDA.bottom <= 0)
		{
			prc->bottom += rcSDA.bottom;
		}
		else
		{
			prc->bottom = rcSDA.bottom;
		}
		if (rcSDA.right <= 0)
		{
			prc->right += rcSDA.right;
		}
		else
		{
			prc->right = rcSDA.right;
		}

#ifdef _DEBUG

		sprintf(szdebugBuffer, "left: %i top: %i right: %i bottom: %i",
		        prc->left, prc->top, prc->right, prc->bottom);
		_LSDEBUG2("Multiple SDA", szdebugBuffer);
#endif // _DEBUG

		SystemParametersInfo(SPI_SETWORKAREA, 0, (PVOID)prc, SPIF_SENDCHANGE);
		//SystemParametersInfo(SPI_GETWORKAREA, 0, (PVOID)prc, SPIF_SENDCHANGE);
		numMonitors++;
	}

	return true;
}


// Reset the desktop area for each monitor to the max
BOOL CALLBACK DesktopAreaResetEnumProc(HMONITOR hMonitor, HDC gdc, LPRECT prc, LPARAM lParam)
{
#ifdef _DEBUG
	sprintf(szdebugBuffer, "left: %i top: %i right: %i bottom: %i",
	        prc->left, prc->top, prc->right, prc->bottom);
	_LSDEBUG2("resetMinMax", szdebugBuffer);
#endif // _DEBUG

	SystemParametersInfo(SPI_SETWORKAREA, 0, (PVOID)prc, SPIF_SENDCHANGE);
	//SystemParametersInfo(SPI_GETWORKAREA, 0, (PVOID)prc, SPIF_SENDCHANGE);
	return true;
}


// Set the desktop area for everymonitor but the first to the max
BOOL CALLBACK DesktopAreaEnumProc(HMONITOR hMonitor, HDC gdc, LPRECT prc, LPARAM lParam)
{
#ifdef _DEBUG
	sprintf(szdebugBuffer, "left: %i top: %i right: %i bottom: %i",
	        prc->left, prc->top, prc->right, prc->bottom);
	_LSDEBUG2("setMinMax", szdebugBuffer);
#endif // _DEBUG

	PRECT pr = (PRECT)lParam;
	if (!EqualRect(pr, prc))
	{
		SystemParametersInfo(SPI_SETWORKAREA, 0, (PVOID)prc, SPIF_SENDCHANGE);
	}
	//SystemParametersInfo(SPI_GETWORKAREA, 0, (PVOID)prc, SPIF_SENDCHANGE);
	return true;
}
